@@ -21,11 +21,75 @@ use Travel::Status::DE::HAFAS::Polyline qw(decode_polyline);
 use Travel::Status::DE::HAFAS::Product;
 use Travel::Status::DE::HAFAS::StopFinder;
 
-our $VERSION = '5.06';
+our $VERSION = '6.00';
 
 # {{{ Endpoint Definition
 
 my %hafas_instance = (
+	AVV => {
+		stopfinder  => 'https://auskunft.avv.de/bin/ajax-getstop.exe',
+		mgate       => 'https://auskunft.avv.de/bin/mgate.exe',
+		name        => 'Aachener Verkehrsverbund',
+		productbits => [
+			[ regio    => 'region trains' ],
+			[ ic_ec    => 'long distance trains' ],
+			[ ice      => 'long distance trains' ],
+			[ bus      => 'long distance busses' ],
+			[ s        => 'sububrban trains' ],
+			[ u        => 'underground trains' ],
+			[ tram     => 'trams' ],
+			[ bus      => 'busses' ],
+			[ bus      => 'additional busses' ],
+			[ ondemand => 'on-demand services' ],
+			[ ferry    => 'maritime transit' ]
+		],
+		languages => [qw[de]],
+		request   => {
+			client => {
+				id   => 'AVV_AACHEN',
+				type => 'WEB',
+				name => 'webapp',
+				l    => 'vs_avv',
+			},
+			ver  => '1.26',
+			auth => {
+				type => 'AID',
+				aid  => '4vV1AcH3' . 'N511icH',
+			},
+			lang => 'deu',
+		},
+	},
+	BART => {
+		stopfinder  => 'https://planner.bart.gov/bin/ajax-getstop.exe',
+		mgate       => 'https://planner.bart.gov/bin/mgate.exe',
+		name        => 'Bay Area Rapid Transit',
+		time_zone   => 'America/Los_Angeles',
+		productbits => [
+			[ _     => undef ],
+			[ _     => undef ],
+			[ cc    => 'cable cars' ],
+			[ regio => 'regional trains' ],
+			[ _     => undef ],
+			[ bus   => 'busses' ],
+			[ ferry => 'maritime transit' ],
+			[ bart  => 'BART trains' ],
+			[ tram  => 'trams' ],
+		],
+		languages => [qw[en]],
+		request   => {
+			client => {
+				id   => 'BART',
+				type => 'WEB',
+				name => 'webapp',
+			},
+			ver  => '1.40',
+			auth => {
+				type => 'AID',
+				aid  => 'kEwHkFUC' . 'IL500dym',
+			},
+			lang => 'en',
+		},
+	},
 	DB => {
 		stopfinder    => 'https://reiseauskunft.bahn.de/bin/ajax-getstop.exe',
 		mgate         => 'https://reiseauskunft.bahn.de/bin/mgate.exe',
@@ -50,6 +114,40 @@ my %hafas_instance = (
 			},
 		},
 	},
+	IE => {
+		stopfinder =>
+		  'https://journeyplanner.irishrail.ie/bin/ajax-getstop.exe',
+		mgate       => 'https://journeyplanner.irishrail.ie/bin/mgate.exe',
+		name        => 'Iarnród Éireann',
+		time_zone   => 'Europe/Dublin',
+		productbits => [
+			[ _     => undef ],
+			[ ic    => 'national trains' ],
+			[ _     => undef ],
+			[ regio => 'regional trains' ],
+			[ dart  => 'DART trains' ],
+			[ _     => undef ],
+			[ luas  => 'LUAS trams' ],
+		],
+		languages => [qw[en ga]],
+		request   => {
+			client => {
+				id   => 'IRISHRAIL',
+				type => 'IPA',
+				name => 'IrishRailPROD-APPSTORE',
+				v    => '4000100',
+				os   => 'iOS 12.4.8',
+			},
+			ver  => '1.33',
+			auth => {
+				type => 'AID',
+				aid  => 'P9bplgVCG' . 'nozdgQE',
+			},
+			lang => 'en',
+		},
+		salt   => 'i5s7m3q9' . 'z6b4k1c2',
+		micmac => 1,
+	},
 	NAHSH => {
 		mgate       => 'https://nah.sh.hafas.de/bin/mgate.exe',
 		stopfinder  => 'https://nah.sh.hafas.de/bin/ajax-getstop.exe',
@@ -119,6 +217,7 @@ my %hafas_instance = (
 		mgate       => 'https://fahrplan.oebb.at/bin/mgate.exe',
 		stopfinder  => 'https://fahrplan.oebb.at/bin/ajax-getstop.exe',
 		name        => 'Österreichische Bundesbahnen',
+		time_zone   => 'Europe/Vienna',
 		productbits => [
 			[ ice_rj => 'long distance trains' ],
 			[ sev    => 'rail replacement service' ],
@@ -232,7 +331,8 @@ sub new {
 		confess("The service '$service' is not supported");
 	}
 
-	my $now  = DateTime->now( time_zone => 'Europe/Berlin' );
+	my $now = DateTime->now( time_zone => $hafas_instance{$service}{time_zone}
+		  // 'Europe/Berlin' );
 	my $self = {
 		active_service => $service,
 		arrivals       => $conf{arrivals},
@@ -245,6 +345,7 @@ sub new {
 		station        => $conf{station},
 		ua             => $ua,
 		now            => $now,
+		tz_offset      => $now->offset / 60,
 	};
 
 	bless( $self, $obj );
@@ -386,7 +487,7 @@ sub new {
 
 	$self->{strptime_obj} //= DateTime::Format::Strptime->new(
 		pattern   => '%Y%m%dT%H%M%S',
-		time_zone => 'Europe/Berlin',
+		time_zone => $hafas_instance{$service}{time_zone} // 'Europe/Berlin',
 	);
 
 	my $json = $self->{json} = JSON->new->utf8;
@@ -830,16 +931,30 @@ sub parse_board {
 	my @jnyL = @{ $self->{raw_json}{svcResL}[0]{res}{jnyL} // [] };
 
 	for my $result (@jnyL) {
-		push(
-			@{ $self->{results} },
-			Travel::Status::DE::HAFAS::Journey->new(
-				common  => $self->{raw_json}{svcResL}[0]{res}{common},
-				prodL   => $prodL,
-				locL    => \@locL,
-				journey => $result,
-				hafas   => $self,
-			)
-		);
+		eval {
+			push(
+				@{ $self->{results} },
+				Travel::Status::DE::HAFAS::Journey->new(
+					common  => $self->{raw_json}{svcResL}[0]{res}{common},
+					prodL   => $prodL,
+					locL    => \@locL,
+					journey => $result,
+					hafas   => $self,
+				)
+			);
+		};
+		if ($@) {
+			if ( $@ =~ m{Invalid local time for date in time zone} ) {
+
+				# Yes, HAFAS does in fact return invalid times during DST change
+				# (as in, it returns 02:XX:XX timestamps when the time jumps from 02:00:00 to 03:00:00)
+				# It's not clear what exactly is going wrong where and whether a 2:30 or a 3:30 journey is the correct one.
+				# For now, silently discard the affected journeys.
+			}
+			else {
+				warn("Skipping $result->{jid}: $@");
+			}
+		}
 	}
 	return $self;
 }
@@ -1025,7 +1140,7 @@ monitors
 
 =head1 VERSION
 
-version 5.06
+version 6.00
 
 =head1 DESCRIPTION
 
@@ -1260,17 +1375,42 @@ arguments. Otherwise, the caller must specify B<promise> and B<user_agent>
 =item $status->get_active_service
 
 Returns a hashref describing the active service when a service is active and
-nothing otherwise. The hashref contains the keys B<url> (URL to the station
-board service), B<stopfinder> (URL to the stopfinder service, if supported),
-B<name>, and B<productbits> (arrayref describing the supported modes of
-transport, may contain duplicates).
+nothing otherwise. The hashref contains the following keys.
+
+=over
+
+=item B<name> => I<string>
+
+service name, e.g. Bay Area Rapid Transit or Deutsche Bahn.
+
+=item B<mgate> => I<string>
+
+HAFAS backend URL
+
+=item B<languages> => I<arrayref>
+
+Languages supported by the backend; see the constructor's B<language> argument.
+
+=item B<productbits> => I<arrayref>
+
+MOT bits supported by the backend. I<arrayref> contains either strings
+(one string per mode of transit) or arrayrefs (one string pair per mode of
+transit, with the first entry referring to the MOT identifier and the second
+one containing a slightly longer description of it).
+
+=item B<time_zone> => I<string> (optional)
+
+The time zone this service reports arrival/departure times in. If this key is
+not present, it is safe to assume that it uses Europe/Berlin.
+
+=back
 
 =item Travel::Status::DE::HAFAS::get_services()
 
 Returns an array containing all supported HAFAS services. Each element is a
 hashref and contains all keys mentioned in B<get_active_service>.
 It also contains a B<shortname> key, which is the service name used by
-the constructor's B<service> parameter.
+the constructor's B<service> parameter, e.g. BART or DB.
 
 =item Travel::Status::DE::HAFAS::get_service(I<$service>)
 
