@@ -11,10 +11,10 @@ use DateTime::Format::Strptime;
 use List::Util qw(any);
 use Travel::Status::DE::HAFAS::Stop;
 
-our $VERSION = '5.06';
+our $VERSION = '6.00';
 
 Travel::Status::DE::HAFAS::Journey->mk_ro_accessors(
-	qw(datetime sched_datetime rt_datetime
+	qw(datetime sched_datetime rt_datetime tz_offset
 	  is_additional is_cancelled is_partially_cancelled
 	  station station_eva platform sched_platform rt_platform operator
 	  product
@@ -94,7 +94,8 @@ sub new {
 		}
 		$datetime_ref = DateTime::Format::Strptime->new(
 			pattern   => $parse_fmt,
-			time_zone => 'Europe/Berlin'
+			time_zone => $hafas->get_active_service->{time_zone}
+			  // 'Europe/Berlin'
 		)->parse_datetime($date_ref);
 	}
 
@@ -173,27 +174,33 @@ sub new {
 		  // $journey->{stbStop}{dPltfR}{txt};
 		$ref->{platform} = $ref->{rt_platform} // $ref->{sched_platform};
 
-		my $time_s
-		  = $journey->{stbStop}{ $hafas->{arrivals} ? 'aTimeS' : 'dTimeS' };
-		my $time_r
-		  = $journey->{stbStop}{ $hafas->{arrivals} ? 'aTimeR' : 'dTimeR' };
+		my $datetime_s = Travel::Status::DE::HAFAS::Stop::handle_day_change(
+			$ref,
+			input =>
+			  $journey->{stbStop}{ $hafas->{arrivals} ? 'aTimeS' : 'dTimeS' },
+			offset => $journey->{stbStop}{
+				$hafas->{arrivals}
+				? 'aTZOffset'
+				: 'dTZOffset'
+			},
+			date     => $date,
+			strp_obj => $hafas->{strptime_obj},
+			ref      => $datetime_ref,
+		);
 
-		for my $timestr ( $time_s, $time_r ) {
-			if ( not defined $timestr ) {
-				next;
-			}
-
-			$timestr = Travel::Status::DE::HAFAS::Stop::handle_day_change(
-				input    => $timestr,
-				date     => $date,
-				strp_obj => $hafas->{strptime_obj},
-				ref      => $datetime_ref,
-			);
-
-		}
-
-		my $datetime_s = $time_s;
-		my $datetime_r = $time_r;
+		my $datetime_r = Travel::Status::DE::HAFAS::Stop::handle_day_change(
+			$ref,
+			input =>
+			  $journey->{stbStop}{ $hafas->{arrivals} ? 'aTimeR' : 'dTimeR' },
+			offset => $journey->{stbStop}{
+				$hafas->{arrivals}
+				? 'aTZOffset'
+				: 'dTZOffset'
+			},
+			date     => $date,
+			strp_obj => $hafas->{strptime_obj},
+			ref      => $datetime_ref,
+		);
 
 		my $delay
 		  = $datetime_r
@@ -395,7 +402,7 @@ journey received by Travel::Status::DE::HAFAS
 
 =head1 VERSION
 
-version 5.06
+version 6.00
 
 =head1 DESCRIPTION
 
@@ -404,7 +411,9 @@ a station-specific arrival/departure obtained by a stationboard query, or a
 train journey that does not belong to a specific station.
 
 stationboard-specific accessors are annotated with "(station only)" and return
-undef for non-station journeys.
+undef for non-station journeys. All date and time entries refer to the
+backend time zone (Europe/Berlin in most cases) and do not take local time
+into account; see B<tz_offset> for the latter.
 
 =head1 METHODS
 
@@ -472,6 +481,13 @@ DateTime object indicating the arrival/departure date and time.
 Real-time data if available, schedule data otherwise.
 undef if neither is available.
 
+=item $journey->tz_offset
+
+Offset between backend time zone (default: Europe/Berlin) and this journey's
+time zone in minutes, if any. For instance, if the backend uses UTC+2 (CEST)
+and the journey uses UTC+1 (IST), tz_offset is -60. Returns undef if both use
+the same time zone (or rather, the same UTC offset).
+
 =item $journey->delay (station only)
 
 Delay in minutes, or undef if it is unknown.
